"""
This command is obsolete in Django>=1.7, because proper support for natural keys was later added.
"""
from __future__ import unicode_literals

import os
import gzip
import zipfile
from itertools import product

from django.conf import settings
from django.core import serializers
from django.core.management.base import BaseCommand, CommandError
from django.core.management.color import no_style
from django.db import connections, DEFAULT_DB_ALIAS, IntegrityError, DatabaseError
# from django.db.models import get_apps # pylint: disable=no-name-in-module
from django.apps import apps
from django.utils.encoding import force_text
from django.utils._os import upath

try:
    import bz2
    has_bz2 = True
except ImportError:
    has_bz2 = False

class Command(BaseCommand):
    help = 'Installs the named fixture(s) in the database.'
    args = "fixture [fixture ...]"

    def add_arguments(self, parser):
        parser.add_argument('args', nargs='+')
        parser.add_argument('--database', action='store', dest='database',
            default=DEFAULT_DB_ALIAS, help='Nominates a specific database to load fixtures into. Defaults to the "default" database.')
        parser.add_argument('--ignorenonexistent', '-i', action='store_true', dest='ignore',
            default=False, help='Ignores entries in the serialized data for fields that do not currently exist on the model.')

    def handle(self, *fixture_labels, **options):

        ignore = options.get('ignore')
        using = options.get('database')

        connection = connections[using]

        if not fixture_labels:
            raise CommandError(
                "No database fixture specified. Please provide the path of at "
                "least one fixture in the command line."
            )

        verbosity = int(options.get('verbosity'))
        show_traceback = options.get('traceback')

        # commit is a stealth option - it isn't really useful as
        # a command line option, but it can be useful when invoking
        # loaddata from within another script.
        # If commit=True, loaddata will use its own transaction;
        # if commit=False, the data load SQL will become part of
        # the transaction in place when loaddata was invoked.
        # commit = options.get('commit', True)

        # Keep a count of the installed objects and fixtures
        fixture_count = 0
        loaded_object_count = 0
        fixture_object_count = 0
        models = set()

        humanize = lambda dirname: "'%s'" % dirname if dirname else 'absolute path'

        # Get a cursor (even though we don't need one yet). This has
        # the side effect of initializing the test database (if
        # it isn't already initialized).
        cursor = connection.cursor()

        # Start transaction management. All fixtures are installed in a
        # single transaction to ensure that all references are resolved.
        # if commit:
            # # transaction.commit_unless_managed(using=using)
            # try:
                # transaction.commit(using=using)
            # except transaction.TransactionManagementError:
                # pass
            # # transaction.enter_transaction_management(using=using)
            # transaction.managed(True, using=using)

        class SingleZipReader(zipfile.ZipFile):
            def __init__(self, *args, **kwargs):
                zipfile.ZipFile.__init__(self, *args, **kwargs)
                if settings.DEBUG:
                    assert len(self.namelist()) == 1, \
                        "Zip-compressed fixtures must contain only one file."
            def read(self):
                return zipfile.ZipFile.read(self, self.namelist()[0])

        compression_types = {
            None:   open,
            'gz':   gzip.GzipFile,
            'zip':  SingleZipReader
        }
        if has_bz2:
            compression_types['bz2'] = bz2.BZ2File

        app_module_paths = []
        # for app in get_apps():
        for app_config in apps.get_app_configs():
            # print(app_config.__dict__)
            if hasattr(app_config, 'path'):
                # It's a 'models/' subpackage
                app_module_paths.append(upath(app_config.path))
            else:
                # It's a models.py module
                app_module_paths.append(upath(app_config.__file__))

        app_fixtures = [
            os.path.join(os.path.dirname(path), 'fixtures')
            for path in app_module_paths
        ]

        with connection.constraint_checks_disabled():
            for fixture_label in fixture_labels:
                parts = fixture_label.split('.')

                if len(parts) > 1 and parts[-1] in compression_types:
                    compression_formats = [parts[-1]]
                    parts = parts[:-1]
                else:
                    compression_formats = compression_types.keys()

                if len(parts) == 1:
                    fixture_name = parts[0]
                    formats = serializers.get_public_serializer_formats()
                else:
                    fixture_name, fmt = '.'.join(parts[:-1]), parts[-1]
                    if fmt in serializers.get_public_serializer_formats():
                        formats = [fmt]
                    else:
                        formats = []

                if formats:
                    if verbosity >= 2:
                        self.stdout.write("Loading '%s' fixtures..." % fixture_name)
                else:
                    raise CommandError(
                        ("Problem installing fixture '%s': %s is not a known serialization format.") % (fixture_name, fmt))

                if os.path.isabs(fixture_name):
                    fixture_dirs = [fixture_name]
                else:
                    fixture_dirs = app_fixtures + list(settings.FIXTURE_DIRS) + ['']

                for fixture_dir in fixture_dirs:
                    if verbosity >= 2:
                        self.stdout.write(
                            "Checking %s for fixtures..." % humanize(fixture_dir))

                    label_found = False
                    for combo in product([using, None], formats, compression_formats):
                        database, fmt, compression_format = combo
                        file_name = '.'.join(
                            p for p in [
                                fixture_name, database, fmt, compression_format
                            ]
                            if p
                        )

                        if verbosity >= 3:
                            self.stdout.write("Trying %s for %s fixture '%s'..." % \
                                (humanize(fixture_dir), file_name, fixture_name))
                        full_path = os.path.join(fixture_dir, file_name)
                        open_method = compression_types[compression_format]
                        try:
                            fixture = open_method(full_path, 'r')
                        except IOError:
                            if verbosity >= 2:
                                self.stdout.write("No %s fixture '%s' in %s." % \
                                    (fmt, fixture_name, humanize(fixture_dir)))
                        else:
                            try:
                                if label_found:
                                    raise CommandError(
                                        "Multiple fixtures named '%s' in %s. Aborting." \
                                            % (fixture_name, humanize(fixture_dir)))

                                fixture_count += 1
                                objects_in_fixture = 0
                                loaded_objects_in_fixture = 0
                                if verbosity >= 2:
                                    self.stdout.write("Installing %s fixture '%s' from %s." % \
                                        (fmt, fixture_name, humanize(fixture_dir)))

                                objects = serializers.deserialize(
                                    fmt, fixture, using=using, ignorenonexistent=ignore)

                                for obj in objects:

                                    try:
                                        # Attempt to lookup any existing object using natural
                                        # keys and use that object's PK to duplicate and
                                        # conflict records aren't created.
                                        nk = obj.object.natural_key()
                                        real_object = type(obj.object).objects\
                                            .get_by_natural_key(*nk)
                                        if real_object:
                                            obj.object.pk = real_object.pk
                                    except AttributeError:
                                        # Model class doesn't support natural keys.
                                        pass
                                    except type(obj.object).DoesNotExist:
                                        # No existing record, so proceed as normal.
                                        pass

                                    objects_in_fixture += 1
                                    # if router.allow_syncdb(using, obj.object.__class__):
                                    loaded_objects_in_fixture += 1
                                    models.add(obj.object.__class__)
                                    try:
                                        obj.save(using=using)
                                    except (DatabaseError, IntegrityError) as e:
                                        e.args = (
                                            ("Could not load %(app_label)s."
                                            "%(object_name)s(pk=%(pk)s): %(error_msg)s") \
                                            % {
                                                'app_label': obj.object._meta.app_label,
                                                'object_name': obj.object._meta.object_name,
                                                'pk': obj.object.pk,
                                                'error_msg': force_text(e)
                                            },)
                                        raise

                                loaded_object_count += loaded_objects_in_fixture
                                fixture_object_count += objects_in_fixture
                                label_found = True
                            except Exception as e:
                                if not isinstance(e, CommandError):
                                    e.args = ("Problem installing fixture '%s': %s" \
                                        % (full_path, e),)
                                raise
                            finally:
                                fixture.close()

                            # If the fixture we loaded contains 0 objects, assume that an
                            # error was encountered during fixture loading.
                            if objects_in_fixture == 0:
                                raise CommandError(
                                    ("No fixture data found for '%s'. "
                                    "(File format may be invalid.)") \
                                        % (fixture_name))

        # Since we disabled constraint checks, we must manually check for
        # any invalid keys that might have been added
        table_names = [model._meta.db_table for model in models]
        try:
            connection.check_constraints(table_names=table_names)
        except Exception as e:
            e.args = ("Problem installing fixtures: %s" % e,)
            raise

        # If we found even one object in a fixture, we need to reset the
        # database sequences.
        if loaded_object_count > 0:
            sequence_sql = connection.ops.sequence_reset_sql(no_style(), models)
            if sequence_sql:
                if verbosity >= 2:
                    self.stdout.write("Resetting sequences\n")
                for line in sequence_sql:
                    cursor.execute(line)

        # if commit:
            # transaction.commit(using=using)
            # transaction.leave_transaction_management(using=using)

        if verbosity >= 1:
            if fixture_object_count == loaded_object_count:
                self.stdout.write("Installed %d object(s) from %d fixture(s)" % (
                    loaded_object_count, fixture_count))
            else:
                self.stdout.write("Installed %d object(s) (of %d) from %d fixture(s)" % (
                    loaded_object_count, fixture_object_count, fixture_count))

        # Close the DB connection. This is required as a workaround for an
        # edge case in MySQL: if the same connection is used to
        # create tables, load data, and query, the query can return
        # incorrect results. See Django #7572, MySQL #37735.
        # if commit:
            # connection.close()



from __future__ import print_function

from django.core.management.base import BaseCommand
try:
    from django.core.cache import caches as _get_cache
except ImportError:
    from django.core.cache import get_cache as _get_cache

def get_cache(name):
    # print('_get_cache:', _get_cache)
    # if isinstance(_get_cache, dict):
    return _get_cache[name]
    # return _get_cache(name)

class Command(BaseCommand):
    args = ''
    help = 'Confirms the cache works.'

    def add_arguments(self, parser):
        parser.add_argument('--name', default='default')
        parser.add_argument('--key', default='key')
        parser.add_argument('--value', default='value')

    def handle(self, *args, **options):

        cache = get_cache(options['name'])
        cache_key = options['key']
        cache_value = options['value']

        cache.delete(cache_key)
        _cache_value = cache.get(cache_key)
        assert _cache_value is None

        cache.set(cache_key, cache_value, 60)
        _cache_value = cache.get(cache_key)
        assert _cache_value == cache_value, \
            'Cache failed. Expected %s but cache gave us %s.' % (repr(cache_value), repr(_cache_value))

        print('Cache succeeded.')
